/*
 * Licensed Materials - Property of IBM pidnumber (C) Copyright IBM Corporation
 * 2005. All Rights Reserved. US Government Users Restricted Rights- Use,
 * duplication or disclosure restricted by GSA ADP Schedule Contract with IBM
 * Corp.
 */
package com.ibm.j2ca.sample.twineball.emd.discovery;

import java.io.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.resource.spi.ResourceAdapterInternalException;
import com.ibm.j2ca.base.WBIManagedConnectionFactory;
import com.ibm.j2ca.extension.emd.PropertyNameHelper;
import com.ibm.j2ca.extension.emd.description.EMDConstants;
import com.ibm.j2ca.extension.emd.discovery.*;
import com.ibm.j2ca.extension.emd.discovery.connection.*;
import com.ibm.j2ca.extension.emd.internal.DisplayNameHelper;
import com.ibm.j2ca.extension.logging.LogUtils;
import com.ibm.j2ca.sample.twineball.emd.Constants;
import com.ibm.j2ca.sample.twineball.emd.TwineBallAdapterType;
import com.ibm.j2ca.sample.twineball.emd.description.TwineBallInboundServiceDescription;
import com.ibm.j2ca.sample.twineball.emd.description.TwineBallOutboundServiceDescription;
import commonj.connector.metadata.MetadataException;
import commonj.connector.metadata.MetadataConfigurationType;
import commonj.connector.metadata.description.ServiceDescription;
import commonj.connector.metadata.discovery.*;
import commonj.connector.metadata.discovery.connection.MetadataConnection;
import commonj.connector.metadata.discovery.properties.PropertyGroup;
import commonj.connector.metadata.discovery.properties.SingleValuedProperty;
import commonj.connector.tool.ToolContext;

/**
 * This is the main entry point for the metadata discovery.  We'll set up the adapter type here and work from there.
 */
public class TwineBallMetadataDiscovery extends WBIMetadataDiscoveryImpl {

    private static final String CURRENT_PACKAGE = "com.ibm.j2ca.sample.twineball.emd";
    private static WBIAdapterTypeImpl adapterType;
    WBIMetadataConnectionImpl connection;
    TwineBallMetadataTree tree;
    static File mylogfile = null;
    static FileOutputStream outStream = null;
    static PrintWriter logWriter = null;
    private WBIOutboundConnectionTypeImpl connType;
    Logger logger;
    private ToolContext context;
    private static final String CLASSNAME = "TwineBallMetadataDiscovery";
    private static final String GETTOOLCONTEXT = "getToolContext";
    PropertyNameHelper helper;

    private LogUtils logUtils;


    /**
     *
     */
    public TwineBallMetadataDiscovery() throws MetadataException {
        super(CURRENT_PACKAGE);
	helper = new PropertyNameHelper(null, CURRENT_PACKAGE, null);
    }

    /*
     * (non-Javadoc)
     *
     * @see commonj.connector.metadata.discovery.MetadataDiscovery#getAdapterTypeSummaries()
     */
    public AdapterTypeSummary[] getAdapterTypeSummaries() throws MetadataException {
        if(adapterType == null){
        	adapterType = new TwineBallAdapterType(helper);
        }
    	if(logUtils != null){
    		this.logUtils.trace(Level.FINEST, CLASSNAME, "getAdapterTypeSummaries", "Enter Method");
    	}
        return new WBIAdapterTypeImpl[] { adapterType };
    }

    /**
     * @see commonj.connector.metadata.discovery.MetadataDiscovery#getAdapterType(java.lang.String)
     */
    public AdapterType getAdapterType(String adapterTypeID) throws MetadataException {
        if (adapterTypeID.equals("TwineBall Resource Adapter")) {
            return adapterType;
        } else
            throw new MetadataException("Wrong adapterTypeID, 'TwineBall ResourceAdapter' expected, received: " + adapterTypeID);
    }

    /**
     * @see commonj.connector.metadata.discovery.MetadataDiscovery#getMetadataTree(commonj.connector.metadata.discovery.connection.MetadataConnection)
     */
    public MetadataTree getMetadataTree(MetadataConnection conn) throws MetadataException {
        this.connection = (WBIMetadataConnectionImpl) conn;
        this.connType = (WBIOutboundConnectionTypeImpl)connection.getConnectionType();
        if(helper != null){
		helper.setMetadataConfiguration(super.getConfiguration());
        	tree = new TwineBallMetadataTree(connection, helper);

        }
        tree.setSelectionStyle(MetadataTree.MULTI_SELECT);
        return tree;
    }

    /**
     * @see commonj.connector.metadata.discovery.MetadataDiscovery#createServiceDescription(commonj.connector.metadata.discovery.connection.MetadataConnection,
     *      java.lang.String, java.lang.String,
     *      commonj.connector.metadata.discovery.MetadataSelection,
     *      commonj.connector.metadata.discovery.ToolEnvironmentContext)
     */
    public ServiceDescription createServiceDescription(MetadataSelection importSelection) throws MetadataException {
        ServiceDescription description = null;

        WBIMetadataSelectionImpl selection = (WBIMetadataSelectionImpl) importSelection;

        MetadataImportConfiguration[] confArray = selection.getSelection();

        if (confArray.length == 0)
            return description;
        if(logUtils != null){
    		this.logUtils.trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Number of MetadataImportConfigurations " + confArray.length);
        }
        PropertyGroup serviceType = selection.getAppliedSelectionProperties();

        if(logUtils != null){
    		this.logUtils.trace(Level.ALL, CLASSNAME, "createServiceDescription()", "serviceType ="+serviceType);
        }

        String directionality = getDirectionality(serviceType);

        String nameSpace = getNameSpace(serviceType);

        boolean inbound = false;
        if (directionality.equals(MetadataConfigurationType.INBOUND_SERVICE.toString())) {
            inbound = true;
        }
        if (inbound) {
            description = createInboundServiceDescription(importSelection, selection, nameSpace);
        } else {
            description = createOutboundServiceDescription(importSelection, selection, nameSpace);
        }
        return description;
    }

    /**
     * @param serviceType
     * @return
     */
    private String getDirectionality(PropertyGroup serviceType) {

      SingleValuedProperty prop = (SingleValuedProperty) serviceType.getProperty("ServiceType");

	String propValue = ""; //$NON-NLS-1$ 
	if (prop.getValue() != null && !prop.getValue().equals(""))
			propValue = (String) prop.getValue();
	else
			propValue = (String) prop.getPropertyType().getDefaultValue();

        return propValue;
    }

    /**
     * @param serviceType
     * @return
     */
    private String getNameSpace(PropertyGroup serviceType) {
        SingleValuedProperty prop;
        prop = (SingleValuedProperty) serviceType.getProperty("NameSpace");
        String nameSpace = (String) prop.getValue();
        if (nameSpace == null) nameSpace = Constants.TB_DEFAULT_NAMESPACE;
        return nameSpace;
    }

    /**
     * @param importSelection
     * @param selection
     * @param nameSpace
     * @return
     * @throws MetadataException
     */
    private ServiceDescription createOutboundServiceDescription(MetadataSelection importSelection, WBIMetadataSelectionImpl selection, String nameSpace) throws MetadataException {
        ServiceDescription description;
        TwineBallOutboundServiceDescription outboundDescription = null;
        if(helper != null){
        	outboundDescription = new TwineBallOutboundServiceDescription(helper);
        }
        String interfaceName = "TwineBallOutboundInterface";
        outboundDescription.setName(interfaceName);
        outboundDescription.setNameSpace(nameSpace);
        if(logUtils != null){
   		 	logUtils.trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Setting function Descriptions");
        }
        outboundDescription.setFunctionDescriptions(importSelection);
        if(logUtils != null){
   		 	logUtils.trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Setting MetadataConnectionConfiguration");
        }
        outboundDescription.setMetadataConnectionConfiguration(connType.createOutboundConnectionConfiguration());
        outboundDescription.setComment("Service Description for Outbound");
        outboundDescription.setMetadataSelectionProperties(selection.getAppliedSelectionProperties());
        WBIOutboundConnectionConfigurationImpl outboundConf = (WBIOutboundConnectionConfigurationImpl) adapterType.getOutboundConnectionTypes()[1].createOutboundConnectionConfiguration();
        WBIManagedConnectionFactory mcf = (WBIManagedConnectionFactory)connType.getManagedConnectionFactoryJavaBean();
        mcf.set_LogUtils(null);
        outboundConf.copyPropertyValues(((WBIOutboundConnectionConfigurationImpl) connType.createOutboundConnectionConfiguration()).getAppliedProperties());
        description = outboundDescription;
        outboundDescription.setOutboundConnectionAdvancedConfiguration(outboundConf);
        if(logUtils != null){
   		 	logUtils.trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Service Description Created Successfully");
        }
        return description;
    }


    /**
     * @param importSelection
     * @param selection
     * @param nameSpace
     * @return
     * @throws MetadataException
     */
    private ServiceDescription createInboundServiceDescription(MetadataSelection importSelection, WBIMetadataSelectionImpl selection, String nameSpace) throws MetadataException {
    	TwineBallInboundServiceDescription inboundDescription = null;
    	if(helper != null){
    		inboundDescription = new TwineBallInboundServiceDescription(helper,this);
    	}
        String serviceName = "TwineBallInboundInterface";
        inboundDescription.setName(serviceName);
        inboundDescription.setNameSpace(nameSpace);
        if(logUtils != null){
   		 	logUtils.trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Setting function Descriptions");
        }
        inboundDescription.setFunctionDescriptions(importSelection);
        WBIInboundConnectionConfigurationImpl inboundConf = (WBIInboundConnectionConfigurationImpl) adapterType.getInboundConnectionTypes()[0].createInboundConnectionConfiguration();
        inboundConf.copyPropertyValues(((WBIOutboundConnectionConfigurationImpl) connType.createOutboundConnectionConfiguration()).getAppliedProperties());
        if(logUtils != null){
   		 	logUtils.trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Setting InboundAdvancedConnectionConfiguration");
        }
        inboundDescription.setInboundConnectionAdvancedConfiguration(inboundConf);
        if(logUtils != null){
   		 	logUtils.trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Setting MetadataConnectionConfiguration");
        }
        inboundDescription.setMetadataConnectionConfiguration(connType.createOutboundConnectionConfiguration());
        inboundDescription.setComment("Service Description for Inbound");
        inboundDescription.setMetadataSelectionProperties(selection.getAppliedSelectionProperties());
        if(logUtils != null){
   		 	logUtils.trace(Level.FINER, CLASSNAME, "createServiceDescription()", "Service Description Created Successfully");
        }
        return inboundDescription;
    }

    public void setToolContext(ToolContext toolContext) {
    	super.setToolContext(toolContext);
        try {
            LogUtils logUtils = new LogUtils(toolContext.getLogger(), CURRENT_PACKAGE, "TwineBall Adapter", "6.0.0");
            if(logUtils != null) {
            	logUtils.trace(Level.FINER, CLASSNAME, "getToolContext()", "LogUtils Initilialized");
            	super.setLogUtils(logUtils);
            }
            this.helper = new PropertyNameHelper(logUtils, CURRENT_PACKAGE,toolContext );
            this.helper.setLogUtils(logUtils);
        } catch (ResourceAdapterInternalException e) {
            e.printStackTrace();
            System.out.println(e.getMessage());
            System.out.println("Unable to create LogUtils instance");
        } catch (MetadataException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
	/**
	 * @return Returns the adapterType.
	 */
	public static WBIAdapterTypeImpl getAdapterType() {
		return adapterType;
	}
	/**
	 * @param adapterType The adapterType to set.
	 */
	public static void setAdapterType(WBIAdapterTypeImpl adapterType) {
		TwineBallMetadataDiscovery.adapterType = adapterType;
	}

	public MetadataConfigurationType[] getSupportedConfiguration() {
		return getMetadataConfiguration();
	}

	public static MetadataConfigurationType[] getMetadataConfiguration() {
		MetadataConfigurationType[] array = new MetadataConfigurationType[5];
		array[0] = MetadataConfigurationType.GENERATED_DATA_BINDING;
		array[1] = MetadataConfigurationType.GENERIC_RECORDS;
		array[2] = MetadataConfigurationType.GENERATED_RECORDS;
		array[3] = MetadataConfigurationType.INBOUND_SERVICE;
		array[4] = MetadataConfigurationType.OUTBOUND_SERVICE;
		return array;
	}

  }